@@ -15,7 +15,6 @@
 
 import copy
 import datetime
-import json
 import os
 
 from oslo_config import cfg
@@ -25,7 +24,6 @@ from oslo_utils import timeutils
 
 from ironic_inspector.common.i18n import _
 from ironic_inspector.common import ironic as ir_utils
-from ironic_inspector.common import swift
 from ironic_inspector import introspection_state as istate
 from ironic_inspector import node_cache
 from ironic_inspector.plugins import base as plugins_base
@@ -38,7 +36,6 @@ CONF = cfg.CONF
 LOG = utils.getProcessingLogger(__name__)
 
 _STORAGE_EXCLUDED_KEYS = {'logs'}
-_UNPROCESSED_DATA_STORE_SUFFIX = 'UNPROCESSED'
 
 
 def _store_logs(introspection_data, node_info):
@@ -143,48 +140,44 @@ def _filter_data_excluded_keys(data):
             if k not in _STORAGE_EXCLUDED_KEYS}
 
 
-def _store_data(node_info, data, suffix=None):
-    if CONF.processing.store_data != 'swift':
-        LOG.debug("Swift support is disabled, introspection data "
-                  "won't be stored", node_info=node_info)
-        return
+def store_introspection_data(node_uuid, data, processed=True):
+    """Store introspection data to the storage backend.
 
-    swift_object_name = swift.store_introspection_data(
-        _filter_data_excluded_keys(data),
-        node_info.uuid,
-        suffix=suffix
-    )
-    LOG.info('Introspection data was stored in Swift in object '
-             '%s', swift_object_name, node_info=node_info)
-    if CONF.processing.store_data_location:
-        node_info.patch([{'op': 'add', 'path': '/extra/%s' %
-                          CONF.processing.store_data_location,
-                          'value': swift_object_name}])
+    :param node_uuid: node UUID
+    :param data: Introspection data to be saved
+    :param processed: The type of introspection data, set to True means the
+                      introspection data is processed, otherwise unprocessed.
+    :raises: utils.Error
+    """
+    introspection_data_manager = plugins_base.introspection_data_manager()
+    store = CONF.processing.store_data
+    ext = introspection_data_manager[store].obj
+    ext.save(node_uuid, data, processed)
 
 
-def _store_unprocessed_data(node_info, data):
+def _store_unprocessed_data(node_uuid, data):
     # runs in background
     try:
-        _store_data(node_info, data,
-                    suffix=_UNPROCESSED_DATA_STORE_SUFFIX)
+        store_introspection_data(node_uuid, data, processed=False)
     except Exception:
         LOG.exception('Encountered exception saving unprocessed '
-                      'introspection data', node_info=node_info,
-                      data=data)
+                      'introspection data for node %s', node_uuid, data=data)
 
 
-def _get_unprocessed_data(uuid):
-    if CONF.processing.store_data == 'swift':
-        LOG.debug('Fetching unprocessed introspection data from '
-                  'Swift for %s', uuid)
-        return json.loads(
-            swift.get_introspection_data(
-                uuid,
-                suffix=_UNPROCESSED_DATA_STORE_SUFFIX
-            )
-        )
-    else:
-        raise utils.Error(_('Swift support is disabled'), code=400)
+def get_introspection_data(uuid, processed=True, get_json=False):
+    """Get introspection data from the storage backend.
+
+    :param uuid: node UUID
+    :param processed: Indicates the type of introspection data to be read,
+                      set True to request processed introspection data.
+    :param get_json: Specify whether return the introspection data in json
+                     format, string value is returned if False.
+    :raises: utils.Error
+    """
+    introspection_data_manager = plugins_base.introspection_data_manager()
+    store = CONF.processing.store_data
+    ext = introspection_data_manager[store].obj
+    return ext.get(uuid, processed=processed, get_json=get_json)
 
 
 def process(introspection_data):
@@ -205,7 +198,7 @@ def process(introspection_data):
         msg = _('The following failures happened during running '
                 'pre-processing hooks:\n%s') % '\n'.join(failures)
         if node_info is not None:
-            node_info.finished(error='\n'.join(failures))
+            node_info.finished(istate.Events.error, error='\n'.join(failures))
         _store_logs(introspection_data, node_info)
         raise utils.Error(msg, node_info=node_info, data=introspection_data)
 
@@ -221,20 +214,20 @@ def process(introspection_data):
     # Note(mkovacik): store data now when we're sure that a background
     # thread won't race with other process() or introspect.abort()
     # call
-    utils.executor().submit(_store_unprocessed_data, node_info,
+    utils.executor().submit(_store_unprocessed_data, node_info.uuid,
                             unprocessed_data)
 
     try:
         node = node_info.node()
     except ir_utils.NotFound as exc:
         with excutils.save_and_reraise_exception():
-            node_info.finished(error=str(exc))
+            node_info.finished(istate.Events.error, error=str(exc))
             _store_logs(introspection_data, node_info)
 
     try:
         result = _process_node(node_info, node, introspection_data)
     except utils.Error as exc:
-        node_info.finished(error=str(exc))
+        node_info.finished(istate.Events.error, error=str(exc))
         with excutils.save_and_reraise_exception():
             _store_logs(introspection_data, node_info)
     except Exception as exc:
@@ -242,7 +235,7 @@ def process(introspection_data):
         msg = _('Unexpected exception %(exc_class)s during processing: '
                 '%(error)s') % {'exc_class': exc.__class__.__name__,
                                 'error': exc}
-        node_info.finished(error=msg)
+        node_info.finished(istate.Events.error, error=msg)
         _store_logs(introspection_data, node_info)
         raise utils.Error(msg, node_info=node_info, data=introspection_data,
                           code=500)
@@ -266,7 +259,7 @@ def _process_node(node_info, node, introspection_data):
     # NOTE(dtantsur): repeat the check in case something changed
     ir_utils.check_provision_state(node)
     _run_post_hooks(node_info, introspection_data)
-    _store_data(node_info, introspection_data)
+    store_introspection_data(node_info.uuid, introspection_data)
 
     ironic = ir_utils.get_client()
     pxe_filter.driver().sync(ironic)
@@ -282,7 +275,7 @@ def _process_node(node_info, node, introspection_data):
     return resp
 
 
-@node_cache.fsm_transition(istate.Events.finish)
+@node_cache.triggers_fsm_error_transition()
 def _finish(node_info, ironic, introspection_data, power_off=True):
     if power_off:
         LOG.debug('Forcing power off of node %s', node_info.uuid)
@@ -299,31 +292,30 @@ def _finish(node_info, ironic, introspection_data, power_off=True):
                          'its power management configuration: '
                          '%(exc)s') % {'node': node_info.uuid, 'exc':
                                        exc})
-                node_info.finished(error=msg)
                 raise utils.Error(msg, node_info=node_info,
                                   data=introspection_data)
         LOG.info('Node powered-off', node_info=node_info,
                  data=introspection_data)
 
-    node_info.finished()
+    node_info.finished(istate.Events.finish)
     LOG.info('Introspection finished successfully',
              node_info=node_info, data=introspection_data)
 
 
-def reapply(node_ident):
+def reapply(node_uuid, data=None):
     """Re-apply introspection steps.
 
     Re-apply preprocessing, postprocessing and introspection rules on
     stored data.
 
-    :param node_ident: node UUID or name
+    :param node_uuid: node UUID
+    :param data: unprocessed introspection data to be reapplied
     :raises: utils.Error
-
     """
 
     LOG.debug('Processing re-apply introspection request for node '
-              'UUID: %s', node_ident)
-    node_info = node_cache.get_node(node_ident, locked=False)
+              'UUID: %s', node_uuid)
+    node_info = node_cache.get_node(node_uuid, locked=False)
     if not node_info.acquire_lock(blocking=False):
         # Note (mkovacik): it should be sufficient to check data
         # presence & locking. If either introspection didn't start
@@ -332,24 +324,13 @@ def reapply(node_ident):
         raise utils.Error(_('Node locked, please, try again later'),
                           node_info=node_info, code=409)
 
-    utils.executor().submit(_reapply, node_info)
+    utils.executor().submit(_reapply, node_info, introspection_data=data)
 
 
-def _reapply(node_info):
+def _reapply(node_info, introspection_data=None):
     # runs in background
-    try:
-        node_info.started_at = timeutils.utcnow()
-        node_info.commit()
-        introspection_data = _get_unprocessed_data(node_info.uuid)
-    except Exception as exc:
-        LOG.exception('Encountered exception while fetching '
-                      'stored introspection data',
-                      node_info=node_info)
-        msg = (_('Unexpected exception %(exc_class)s while fetching '
-                 'unprocessed introspection data from Swift: %(error)s') %
-               {'exc_class': exc.__class__.__name__, 'error': exc})
-        node_info.finished(error=msg)
-        return
+    node_info.started_at = timeutils.utcnow()
+    node_info.commit()
 
     try:
         ironic = ir_utils.get_client()
@@ -357,14 +338,15 @@ def _reapply(node_info):
         msg = _('Encountered an exception while getting the Ironic client: '
                 '%s') % exc
         LOG.error(msg, node_info=node_info, data=introspection_data)
-        node_info.fsm_event(istate.Events.error)
-        node_info.finished(error=msg)
+        node_info.finished(istate.Events.error, error=msg)
         return
 
     try:
         _reapply_with_data(node_info, introspection_data)
     except Exception as exc:
-        node_info.finished(error=str(exc))
+        msg = (_('Failed reapply for node %(node)s, Error: '
+                 '%(exc)s') % {'node': node_info.uuid, 'exc': exc})
+        LOG.error(msg, node_info=node_info, data=introspection_data)
         return
 
     _finish(node_info, ironic, introspection_data,
@@ -385,6 +367,6 @@ def _reapply_with_data(node_info, introspection_data):
                           '\n'.join(failures), node_info=node_info)
 
     _run_post_hooks(node_info, introspection_data)
-    _store_data(node_info, introspection_data)
+    store_introspection_data(node_info.uuid, introspection_data)
     node_info.invalidate_cache()
     rules.apply(node_info, introspection_data)
