@@ -57,6 +57,7 @@ driver = noop
 debug = True
 introspection_delay = 0
 auth_strategy=noauth
+transport_url=fake://
 [database]
 connection = sqlite:///%(db_file)s
 [processing]
@@ -136,6 +137,13 @@ class Base(base.NodeTest):
         conf_file = get_test_conf_file()
         self.cfg.set_config_files([conf_file])
 
+        # FIXME(milan) FakeListener.poll calls time.sleep() which leads to
+        # busy polling with no sleep at all, effectively blocking the whole
+        # process by consuming all CPU cycles in a single thread. MonkeyPatch
+        # with eventlet.sleep seems to help this.
+        self.useFixture(fixtures.MonkeyPatch(
+            'oslo_messaging._drivers.impl_fake.time.sleep', eventlet.sleep))
+
     def tearDown(self):
         super(Base, self).tearDown()
         node_cache._delete_node(self.uuid)
@@ -160,9 +168,11 @@ class Base(base.NodeTest):
                 raise AssertionError(msg)
         return res
 
-    def call_introspect(self, uuid, **kwargs):
+    def call_introspect(self, uuid, manage_boot=True, **kwargs):
         endpoint = '/v1/introspection/%s' % uuid
-        return self.call('post', endpoint, **kwargs)
+        if manage_boot is not None:
+            endpoint = '%s?manage_boot=%s' % (endpoint, manage_boot)
+        return self.call('post', endpoint)
 
     def call_get_status(self, uuid, **kwargs):
         return self.call('get', '/v1/introspection/%s' % uuid, **kwargs).json()
@@ -250,6 +260,7 @@ class Test(Base):
         self.cli.port.create.assert_called_once_with(
             node_uuid=self.uuid, address='11:22:33:44:55:66', extra={},
             pxe_enabled=True)
+        self.assertTrue(self.cli.node.set_boot_device.called)
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=True, state=istate.States.finished)
@@ -351,6 +362,24 @@ class Test(Base):
                              status_.get('links')[0].get('href')).path).json()
                           for status_ in statuses])
 
+    def test_manage_boot(self):
+        self.call_introspect(self.uuid, manage_boot=False)
+        eventlet.greenthread.sleep(DEFAULT_SLEEP)
+        self.assertFalse(self.cli.node.set_power_state.called)
+
+        status = self.call_get_status(self.uuid)
+        self.check_status(status, finished=False, state=istate.States.waiting)
+
+        res = self.call_continue(self.data)
+        self.assertEqual({'uuid': self.uuid}, res)
+        eventlet.greenthread.sleep(DEFAULT_SLEEP)
+
+        self.cli.node.update.assert_called_with(self.uuid, mock.ANY)
+        self.assertFalse(self.cli.node.set_boot_device.called)
+
+        status = self.call_get_status(self.uuid)
+        self.check_status(status, finished=True, state=istate.States.finished)
+
     def test_rules_api(self):
         res = self.call_list_rules()
         self.assertEqual([], res)
@@ -605,6 +634,14 @@ class Test(Base):
         self.assertEqual(store_processing_call,
                          store_mock.call_args_list[-1])
 
+        # Reapply with provided data
+        res = self.call_reapply(self.uuid, data=copy.deepcopy(self.data))
+        self.assertEqual(202, res.status_code)
+        self.assertEqual('', res.text)
+        eventlet.greenthread.sleep(DEFAULT_SLEEP)
+
+        self.check_status(status, finished=True, state=istate.States.finished)
+
     @mock.patch.object(swift, 'store_introspection_data', autospec=True)
     @mock.patch.object(swift, 'get_introspection_data', autospec=True)
     def test_edge_state_transitions(self, get_mock, store_mock):
