@@ -15,18 +15,22 @@ import datetime
 import json
 import unittest
 
+import fixtures
 import mock
+import oslo_messaging as messaging
 from oslo_utils import uuidutils
 
 from ironic_inspector.common import ironic as ir_utils
+from ironic_inspector.common import rpc
+from ironic_inspector.common import swift
 import ironic_inspector.conf
 from ironic_inspector.conf import opts as conf_opts
-from ironic_inspector import introspect
 from ironic_inspector import introspection_state as istate
 from ironic_inspector import main
 from ironic_inspector import node_cache
 from ironic_inspector.plugins import base as plugins_base
 from ironic_inspector.plugins import example as example_plugin
+from ironic_inspector.plugins import introspection_data as intros_data_plugin
 from ironic_inspector import process
 from ironic_inspector import rules
 from ironic_inspector.test import base as test_base
@@ -49,36 +53,76 @@ class BaseAPITest(test_base.BaseTest):
 
 
 class TestApiIntrospect(BaseAPITest):
-    @mock.patch.object(introspect, 'introspect', autospec=True)
-    def test_introspect_no_authentication(self, introspect_mock):
+    def setUp(self):
+        super(TestApiIntrospect, self).setUp()
+        self.rpc_get_client_mock = self.useFixture(
+            fixtures.MockPatchObject(rpc, 'get_client', autospec=True)).mock
+        self.client_mock = mock.MagicMock(spec=messaging.RPCClient)
+        self.rpc_get_client_mock.return_value = self.client_mock
+
+    def test_introspect_no_authentication(self):
         CONF.set_override('auth_strategy', 'noauth')
-        res = self.app.post('/v1/introspection/%s' % self.uuid)
-        self.assertEqual(202, res.status_code)
-        introspect_mock.assert_called_once_with(self.uuid,
-                                                token=None)
 
-    @mock.patch.object(introspect, 'introspect', autospec=True)
-    def test_intospect_failed(self, introspect_mock):
-        introspect_mock.side_effect = utils.Error("boom")
         res = self.app.post('/v1/introspection/%s' % self.uuid)
+
+        self.assertEqual(202, res.status_code)
+        self.client_mock.call.assert_called_once_with({}, 'do_introspection',
+                                                      node_id=self.uuid,
+                                                      manage_boot=True,
+                                                      token=None)
+
+    def test_intospect_failed(self):
+        self.client_mock.call.side_effect = utils.Error("boom")
+        res = self.app.post('/v1/introspection/%s' % self.uuid)
+
         self.assertEqual(400, res.status_code)
         self.assertEqual(
             'boom',
             json.loads(res.data.decode('utf-8'))['error']['message'])
-        introspect_mock.assert_called_once_with(
-            self.uuid,
-            token=None)
+        self.client_mock.call.assert_called_once_with({}, 'do_introspection',
+                                                      node_id=self.uuid,
+                                                      manage_boot=True,
+                                                      token=None)
+
+    def test_introspect_no_manage_boot(self):
+        res = self.app.post('/v1/introspection/%s?manage_boot=0' % self.uuid)
+        self.assertEqual(202, res.status_code)
+        self.client_mock.call.assert_called_once_with({}, 'do_introspection',
+                                                      node_id=self.uuid,
+                                                      manage_boot=False,
+                                                      token=None)
+
+    def test_introspect_can_manage_boot_false(self):
+        CONF.set_override('can_manage_boot', False)
+        res = self.app.post('/v1/introspection/%s?manage_boot=0' % self.uuid)
+        self.assertEqual(202, res.status_code)
+        self.client_mock.call.assert_called_once_with({}, 'do_introspection',
+                                                      node_id=self.uuid,
+                                                      manage_boot=False,
+                                                      token=None)
+
+    def test_introspect_can_manage_boot_false_failed(self):
+        CONF.set_override('can_manage_boot', False)
+        res = self.app.post('/v1/introspection/%s' % self.uuid)
+        self.assertEqual(400, res.status_code)
+        self.assertFalse(self.client_mock.call.called)
+
+    def test_introspect_wrong_manage_boot(self):
+        res = self.app.post('/v1/introspection/%s?manage_boot=foo' % self.uuid)
+        self.assertEqual(400, res.status_code)
+        self.assertFalse(self.client_mock.call.called)
+        self.assertEqual(
+            'Invalid boolean value for manage_boot: foo',
+            json.loads(res.data.decode('utf-8'))['error']['message'])
 
     @mock.patch.object(utils, 'check_auth', autospec=True)
-    @mock.patch.object(introspect, 'introspect', autospec=True)
-    def test_introspect_failed_authentication(self, introspect_mock,
-                                              auth_mock):
+    def test_introspect_failed_authentication(self, auth_mock):
         CONF.set_override('auth_strategy', 'keystone')
         auth_mock.side_effect = utils.Error('Boom', code=403)
         res = self.app.post('/v1/introspection/%s' % self.uuid,
                             headers={'X-Auth-Token': 'token'})
         self.assertEqual(403, res.status_code)
-        self.assertFalse(introspect_mock.called)
+        self.assertFalse(self.client_mock.call.called)
 
 
 @mock.patch.object(process, 'process', autospec=True)
@@ -107,45 +151,57 @@ class TestApiContinue(BaseAPITest):
         self.assertFalse(process_mock.called)
 
 
-@mock.patch.object(introspect, 'abort', autospec=True)
 class TestApiAbort(BaseAPITest):
-    def test_ok(self, abort_mock):
-        abort_mock.return_value = '', 202
+    def setUp(self):
+        super(TestApiAbort, self).setUp()
+        self.rpc_get_client_mock = self.useFixture(
+            fixtures.MockPatchObject(rpc, 'get_client', autospec=True)).mock
+        self.client_mock = mock.MagicMock(spec=messaging.RPCClient)
+        self.rpc_get_client_mock.return_value = self.client_mock
+
+    def test_ok(self):
 
         res = self.app.post('/v1/introspection/%s/abort' % self.uuid,
                             headers={'X-Auth-Token': 'token'})
 
-        abort_mock.assert_called_once_with(self.uuid, token='token')
+        self.client_mock.call.assert_called_once_with({}, 'do_abort',
+                                                      node_id=self.uuid,
+                                                      token='token')
         self.assertEqual(202, res.status_code)
         self.assertEqual(b'', res.data)
 
-    def test_no_authentication(self, abort_mock):
-        abort_mock.return_value = b'', 202
+    def test_no_authentication(self):
 
         res = self.app.post('/v1/introspection/%s/abort' % self.uuid)
 
-        abort_mock.assert_called_once_with(self.uuid, token=None)
+        self.client_mock.call.assert_called_once_with({}, 'do_abort',
+                                                      node_id=self.uuid,
+                                                      token=None)
         self.assertEqual(202, res.status_code)
         self.assertEqual(b'', res.data)
 
-    def test_node_not_found(self, abort_mock):
+    def test_node_not_found(self):
         exc = utils.Error("Not Found.", code=404)
-        abort_mock.side_effect = exc
+        self.client_mock.call.side_effect = exc
 
         res = self.app.post('/v1/introspection/%s/abort' % self.uuid)
 
-        abort_mock.assert_called_once_with(self.uuid, token=None)
+        self.client_mock.call.assert_called_once_with({}, 'do_abort',
+                                                      node_id=self.uuid,
+                                                      token=None)
         self.assertEqual(404, res.status_code)
         data = json.loads(str(res.data.decode()))
         self.assertEqual(str(exc), data['error']['message'])
 
-    def test_abort_failed(self, abort_mock):
+    def test_abort_failed(self):
         exc = utils.Error("Locked.", code=409)
-        abort_mock.side_effect = exc
+        self.client_mock.call.side_effect = exc
 
         res = self.app.post('/v1/introspection/%s/abort' % self.uuid)
 
-        abort_mock.assert_called_once_with(self.uuid, token=None)
+        self.client_mock.call.assert_called_once_with({}, 'do_abort',
+                                                      node_id=self.uuid,
+                                                      token=None)
         self.assertEqual(409, res.status_code)
         data = json.loads(res.data.decode())
         self.assertEqual(str(exc), data['error']['message'])
@@ -243,10 +299,9 @@ class TestApiListStatus(GetStatusAPIBaseTest):
 
 
 class TestApiGetData(BaseAPITest):
-    @mock.patch.object(main.swift, 'SwiftAPI', autospec=True)
-    def test_get_introspection_data(self, swift_mock):
-        CONF.set_override('store_data', 'swift', 'processing')
-        data = {
+    def setUp(self):
+        super(TestApiGetData, self).setUp()
+        self.introspection_data = {
             'ipmi_address': '1.2.3.4',
             'cpus': 2,
             'cpu_arch': 'x86_64',
@@ -256,97 +311,88 @@ class TestApiGetData(BaseAPITest):
                 'em1': {'mac': '11:22:33:44:55:66', 'ip': '1.2.0.1'},
             }
         }
+
+    @mock.patch.object(swift, 'SwiftAPI', autospec=True)
+    def test_get_introspection_data_from_swift(self, swift_mock):
+        CONF.set_override('store_data', 'swift', 'processing')
         swift_conn = swift_mock.return_value
-        swift_conn.get_object.return_value = json.dumps(data)
+        swift_conn.get_object.return_value = json.dumps(
+            self.introspection_data)
         res = self.app.get('/v1/introspection/%s/data' % self.uuid)
         name = 'inspector_data-%s' % self.uuid
         swift_conn.get_object.assert_called_once_with(name)
         self.assertEqual(200, res.status_code)
-        self.assertEqual(data, json.loads(res.data.decode('utf-8')))
+        self.assertEqual(self.introspection_data,
+                         json.loads(res.data.decode('utf-8')))
 
-    @mock.patch.object(main.swift, 'SwiftAPI', autospec=True)
-    def test_introspection_data_not_stored(self, swift_mock):
+    @mock.patch.object(intros_data_plugin, 'DatabaseStore',
+                       autospec=True)
+    def test_get_introspection_data_from_db(self, db_mock):
+        CONF.set_override('store_data', 'database', 'processing')
+        db_store = db_mock.return_value
+        db_store.get.return_value = json.dumps(self.introspection_data)
+        res = self.app.get('/v1/introspection/%s/data' % self.uuid)
+        db_store.get.assert_called_once_with(self.uuid, processed=True,
+                                             get_json=False)
+        self.assertEqual(200, res.status_code)
+        self.assertEqual(self.introspection_data,
+                         json.loads(res.data.decode('utf-8')))
+
+    def test_introspection_data_not_stored(self):
         CONF.set_override('store_data', 'none', 'processing')
-        swift_conn = swift_mock.return_value
         res = self.app.get('/v1/introspection/%s/data' % self.uuid)
-        self.assertFalse(swift_conn.get_object.called)
         self.assertEqual(404, res.status_code)
 
     @mock.patch.object(ir_utils, 'get_node', autospec=True)
-    @mock.patch.object(main.swift, 'SwiftAPI', autospec=True)
-    def test_with_name(self, swift_mock, get_mock):
+    @mock.patch.object(main.process, 'get_introspection_data', autospec=True)
+    def test_with_name(self, process_mock, get_mock):
         get_mock.return_value = mock.Mock(uuid=self.uuid)
         CONF.set_override('store_data', 'swift', 'processing')
-        data = {
-            'ipmi_address': '1.2.3.4',
-            'cpus': 2,
-            'cpu_arch': 'x86_64',
-            'memory_mb': 1024,
-            'local_gb': 20,
-            'interfaces': {
-                'em1': {'mac': '11:22:33:44:55:66', 'ip': '1.2.0.1'},
-            }
-        }
-        swift_conn = swift_mock.return_value
-        swift_conn.get_object.return_value = json.dumps(data)
+        process_mock.return_value = json.dumps(self.introspection_data)
         res = self.app.get('/v1/introspection/name1/data')
-        name = 'inspector_data-%s' % self.uuid
-        swift_conn.get_object.assert_called_once_with(name)
         self.assertEqual(200, res.status_code)
-        self.assertEqual(data, json.loads(res.data.decode('utf-8')))
+        self.assertEqual(self.introspection_data,
+                         json.loads(res.data.decode('utf-8')))
         get_mock.assert_called_once_with('name1', fields=['uuid'])
 
 
-@mock.patch.object(process, 'reapply', autospec=True)
 class TestApiReapply(BaseAPITest):
 
     def setUp(self):
         super(TestApiReapply, self).setUp()
+        self.rpc_get_client_mock = self.useFixture(
+            fixtures.MockPatchObject(rpc, 'get_client', autospec=True)).mock
+        self.client_mock = mock.MagicMock(spec=messaging.RPCClient)
+        self.rpc_get_client_mock.return_value = self.client_mock
         CONF.set_override('store_data', 'swift', 'processing')
 
-    def test_ok(self, reapply_mock):
-
+    def test_api_ok(self):
         self.app.post('/v1/introspection/%s/data/unprocessed' %
                       self.uuid)
-        reapply_mock.assert_called_once_with(self.uuid)
+        self.client_mock.call.assert_called_once_with({}, 'do_reapply',
+                                                      node_uuid=self.uuid,
+                                                      data=None)
 
-    def test_user_data(self, reapply_mock):
+    def test_user_data(self):
         res = self.app.post('/v1/introspection/%s/data/unprocessed' %
                             self.uuid, data='some data')
         self.assertEqual(400, res.status_code)
         message = json.loads(res.data.decode())['error']['message']
-        self.assertEqual('User data processing is not supported yet',
-                         message)
-        self.assertFalse(reapply_mock.called)
-
-    def test_swift_disabled(self, reapply_mock):
-        CONF.set_override('store_data', 'none', 'processing')
+        self.assertIn('Invalid data: expected a JSON object', message)
+        self.assertFalse(self.client_mock.call.called)
 
+    def test_user_data_valid(self):
+        data = {"foo": "bar"}
         res = self.app.post('/v1/introspection/%s/data/unprocessed' %
-                            self.uuid)
-        self.assertEqual(400, res.status_code)
-        message = json.loads(res.data.decode())['error']['message']
-        self.assertEqual('Inspector is not configured to store '
-                         'data. Set the [processing] store_data '
-                         'configuration option to change this.',
-                         message)
-        self.assertFalse(reapply_mock.called)
+                            self.uuid, data=json.dumps(data))
+        self.assertEqual(202, res.status_code)
+        self.client_mock.call.assert_called_once_with({}, 'do_reapply',
+                                                      node_uuid=self.uuid,
+                                                      data=data)
 
-    def test_node_locked(self, reapply_mock):
-        exc = utils.Error('Locked.', code=409)
-        reapply_mock.side_effect = exc
-
-        res = self.app.post('/v1/introspection/%s/data/unprocessed' %
-                            self.uuid)
-
-        self.assertEqual(409, res.status_code)
-        message = json.loads(res.data.decode())['error']['message']
-        self.assertEqual(str(exc), message)
-        reapply_mock.assert_called_once_with(self.uuid)
-
-    def test_node_not_found(self, reapply_mock):
-        exc = utils.Error('Not found.', code=404)
-        reapply_mock.side_effect = exc
+    def test_get_introspection_data_error(self):
+        exc = utils.Error('The store is crashed', code=404)
+        self.client_mock.call.side_effect = exc
 
         res = self.app.post('/v1/introspection/%s/data/unprocessed' %
                             self.uuid)
@@ -354,11 +400,13 @@ class TestApiReapply(BaseAPITest):
         self.assertEqual(404, res.status_code)
         message = json.loads(res.data.decode())['error']['message']
         self.assertEqual(str(exc), message)
-        reapply_mock.assert_called_once_with(self.uuid)
+        self.client_mock.call.assert_called_once_with({}, 'do_reapply',
+                                                      node_uuid=self.uuid,
+                                                      data=None)
 
-    def test_generic_error(self, reapply_mock):
+    def test_generic_error(self):
         exc = utils.Error('Oops', code=400)
-        reapply_mock.side_effect = exc
+        self.client_mock.call.side_effect = exc
 
         res = self.app.post('/v1/introspection/%s/data/unprocessed' %
                             self.uuid)
@@ -366,7 +414,19 @@ class TestApiReapply(BaseAPITest):
         self.assertEqual(400, res.status_code)
         message = json.loads(res.data.decode())['error']['message']
         self.assertEqual(str(exc), message)
-        reapply_mock.assert_called_once_with(self.uuid)
+        self.client_mock.call.assert_called_once_with({}, 'do_reapply',
+                                                      node_uuid=self.uuid,
+                                                      data=None)
+
+    @mock.patch.object(ir_utils, 'get_node', autospec=True)
+    def test_reapply_with_node_name(self, get_mock):
+        get_mock.return_value = mock.Mock(uuid=self.uuid)
+        self.app.post('/v1/introspection/%s/data/unprocessed' %
+                      'fake-node')
+        self.client_mock.call.assert_called_once_with({}, 'do_reapply',
+                                                      node_uuid=self.uuid,
+                                                      data=None)
+        get_mock.assert_called_once_with('fake-node', fields=['uuid'])
 
 
 class TestApiRules(BaseAPITest):
@@ -566,8 +626,11 @@ class TestApiVersions(BaseAPITest):
         # API version on unknown pages
         self._check_version_present(self.app.get('/v1/foobar'))
 
+    @mock.patch.object(rpc, 'get_client', autospec=True)
     @mock.patch.object(node_cache, 'get_node', autospec=True)
-    def test_usual_requests(self, get_mock):
+    def test_usual_requests(self, get_mock, rpc_mock):
+        client_mock = mock.MagicMock(spec=messaging.RPCClient)
+        rpc_mock.return_value = client_mock
         get_mock.return_value = node_cache.NodeInfo(uuid=self.uuid,
                                                     started_at=42.0)
         # Successful
@@ -595,6 +658,12 @@ class TestApiVersions(BaseAPITest):
         self.assertIn('%d.%d' % main.MINIMUM_API_VERSION, error)
         self.assertIn('%d.%d' % main.CURRENT_API_VERSION, error)
 
+    def test_request_latest_version(self):
+        headers = {conf_opts.VERSION_HEADER: 'latest'}
+        res = self.app.get('/', headers=headers)
+        self.assertEqual(200, res.status_code)
+        self._check_version_present(res)
+
 
 class TestPlugins(unittest.TestCase):
     @mock.patch.object(example_plugin.ExampleProcessingHook,
@@ -602,7 +671,6 @@ class TestPlugins(unittest.TestCase):
     @mock.patch.object(example_plugin.ExampleProcessingHook,
                        'before_update', autospec=True)
     def test_hook(self, mock_post, mock_pre):
-        plugins_base._HOOKS_MGR = None
         CONF.set_override('processing_hooks', 'example', 'processing')
         mgr = plugins_base.processing_hooks_manager()
         mgr.map_method('before_processing', 'introspection_data')
