@@ -28,11 +28,13 @@ from oslo_utils import uuidutils
 import six
 
 from ironic_inspector.common import ironic as ir_utils
+from ironic_inspector.common import swift
 from ironic_inspector import db
 from ironic_inspector import introspection_state as istate
 from ironic_inspector import node_cache
 from ironic_inspector.plugins import base as plugins_base
 from ironic_inspector.plugins import example as example_plugin
+from ironic_inspector.plugins import introspection_data as intros_data_plugin
 from ironic_inspector import process
 from ironic_inspector.pxe_filter import base as pxe_filter
 from ironic_inspector.test import base as test_base
@@ -74,6 +76,11 @@ class BaseProcessTest(BaseTest):
         self.cli.node.get.return_value = self.node
         self.process_mock = self.process_fixture.mock
         self.process_mock.return_value = self.fake_result_json
+        self.addCleanup(self._cleanup_lock, self.node_info)
+
+    def _cleanup_lock(self, node_info):
+        if node_info._locked:
+            node_info.release_lock()
 
 
 class TestProcess(BaseProcessTest):
@@ -202,7 +209,6 @@ class TestProcess(BaseProcessTest):
         self.assertFalse(self.node_info.finished.called)
 
     def test_error_if_node_not_found_hook(self):
-        plugins_base._NOT_FOUND_HOOK_MGR = None
         self.find_mock.side_effect = utils.NotFoundInCacheError('BOOM')
         self.assertRaisesRegex(utils.Error,
                                'Look up error: BOOM',
@@ -214,7 +220,6 @@ class TestProcess(BaseProcessTest):
 class TestNodeNotFoundHook(BaseProcessTest):
     def test_node_not_found_hook_run_ok(self, hook_mock):
         CONF.set_override('node_not_found_hook', 'example', 'processing')
-        plugins_base._NOT_FOUND_HOOK_MGR = None
         self.find_mock.side_effect = utils.NotFoundInCacheError('BOOM')
         hook_mock.return_value = node_cache.NodeInfo(
             uuid=self.node.uuid,
@@ -225,7 +230,6 @@ class TestNodeNotFoundHook(BaseProcessTest):
 
     def test_node_not_found_hook_run_none(self, hook_mock):
         CONF.set_override('node_not_found_hook', 'example', 'processing')
-        plugins_base._NOT_FOUND_HOOK_MGR = None
         self.find_mock.side_effect = utils.NotFoundInCacheError('BOOM')
         hook_mock.return_value = None
         self.assertRaisesRegex(utils.Error,
@@ -235,7 +239,6 @@ class TestNodeNotFoundHook(BaseProcessTest):
 
     def test_node_not_found_hook_exception(self, hook_mock):
         CONF.set_override('node_not_found_hook', 'example', 'processing')
-        plugins_base._NOT_FOUND_HOOK_MGR = None
         self.find_mock.side_effect = utils.NotFoundInCacheError('BOOM')
         hook_mock.side_effect = Exception('Hook Error')
         self.assertRaisesRegex(utils.Error,
@@ -254,22 +257,13 @@ class TestUnprocessedData(BaseProcessTest):
 
         store_mock.assert_called_once_with(mock.ANY, expected)
 
-    @mock.patch.object(process.swift, 'SwiftAPI', autospec=True)
-    def test_save_unprocessed_data_failure(self, swift_mock):
+    def test_save_unprocessed_data_failure(self):
         CONF.set_override('store_data', 'swift', 'processing')
-        name = 'inspector_data-%s-%s' % (
-            self.uuid,
-            process._UNPROCESSED_DATA_STORE_SUFFIX
-        )
-
-        swift_conn = swift_mock.return_value
-        swift_conn.create_object.side_effect = utils.Error('Oops')
 
         res = process.process(self.data)
 
         # assert store failure doesn't break processing
         self.assertEqual(self.fake_result_json, res)
-        swift_conn.create_object.assert_called_once_with(name, mock.ANY)
 
 
 @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_processing',
@@ -480,8 +474,8 @@ class TestProcessNode(BaseTest):
         finished_mock.assert_called_once_with(
             self.node_info, istate.Events.finish)
 
-    @mock.patch.object(process.swift, 'SwiftAPI', autospec=True)
-    def test_store_data(self, swift_mock):
+    @mock.patch.object(swift, 'SwiftAPI', autospec=True)
+    def test_store_data_with_swift(self, swift_mock):
         CONF.set_override('store_data', 'swift', 'processing')
         swift_conn = swift_mock.return_value
         name = 'inspector_data-%s' % self.uuid
@@ -493,8 +487,8 @@ class TestProcessNode(BaseTest):
         self.assertEqual(expected,
                          json.loads(swift_conn.create_object.call_args[0][1]))
 
-    @mock.patch.object(process.swift, 'SwiftAPI', autospec=True)
-    def test_store_data_no_logs(self, swift_mock):
+    @mock.patch.object(swift, 'SwiftAPI', autospec=True)
+    def test_store_data_no_logs_with_swift(self, swift_mock):
         CONF.set_override('store_data', 'swift', 'processing')
         swift_conn = swift_mock.return_value
         name = 'inspector_data-%s' % self.uuid
@@ -506,23 +500,27 @@ class TestProcessNode(BaseTest):
         self.assertNotIn('logs',
                          json.loads(swift_conn.create_object.call_args[0][1]))
 
-    @mock.patch.object(process.swift, 'SwiftAPI', autospec=True)
-    def test_store_data_location(self, swift_mock):
-        CONF.set_override('store_data', 'swift', 'processing')
-        CONF.set_override('store_data_location', 'inspector_data_object',
-                          'processing')
-        swift_conn = swift_mock.return_value
-        name = 'inspector_data-%s' % self.uuid
-        patch = [{'path': '/extra/inspector_data_object',
-                  'value': name, 'op': 'add'}]
-        expected = self.data
+    @mock.patch.object(node_cache, 'store_introspection_data', autospec=True)
+    def test_store_data_with_database(self, store_mock):
+        CONF.set_override('store_data', 'database', 'processing')
 
         process._process_node(self.node_info, self.node, self.data)
 
-        swift_conn.create_object.assert_called_once_with(name, mock.ANY)
-        self.assertEqual(expected,
-                         json.loads(swift_conn.create_object.call_args[0][1]))
-        self.cli.node.update.assert_any_call(self.uuid, patch)
+        data = intros_data_plugin._filter_data_excluded_keys(self.data)
+        store_mock.assert_called_once_with(self.node_info.uuid, data, True)
+        self.assertEqual(data, store_mock.call_args[0][1])
+
+    @mock.patch.object(node_cache, 'store_introspection_data', autospec=True)
+    def test_store_data_no_logs_with_database(self, store_mock):
+        CONF.set_override('store_data', 'database', 'processing')
+
+        self.data['logs'] = 'something'
+
+        process._process_node(self.node_info, self.node, self.data)
+
+        data = intros_data_plugin._filter_data_excluded_keys(self.data)
+        store_mock.assert_called_once_with(self.node_info.uuid, data, True)
+        self.assertNotIn('logs', store_mock.call_args[0][1])
 
 
 @mock.patch.object(process, '_reapply', autospec=True)
@@ -553,7 +551,8 @@ class TestReapply(BaseTest):
             blocking=False
         )
 
-        reapply_mock.assert_called_once_with(pop_mock.return_value)
+        reapply_mock.assert_called_once_with(pop_mock.return_value,
+                                             introspection_data=None)
 
     @prepare_mocks
     def test_locking_failed(self, pop_mock, reapply_mock):
@@ -567,10 +566,20 @@ class TestReapply(BaseTest):
             blocking=False
         )
 
+    @prepare_mocks
+    def test_reapply_with_data(self, pop_mock, reapply_mock):
+        process.reapply(self.uuid, data=self.data)
+        pop_mock.assert_called_once_with(self.uuid, locked=False)
+        pop_mock.return_value.acquire_lock.assert_called_once_with(
+            blocking=False
+        )
+        reapply_mock.assert_called_once_with(pop_mock.return_value,
+                                             introspection_data=self.data)
+
 
 @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update')
 @mock.patch.object(process.rules, 'apply', autospec=True)
-@mock.patch.object(process.swift, 'SwiftAPI', autospec=True)
+@mock.patch.object(swift, 'SwiftAPI', autospec=True)
 @mock.patch.object(node_cache.NodeInfo, 'finished', autospec=True)
 @mock.patch.object(node_cache.NodeInfo, 'release_lock', autospec=True)
 class TestReapplyNode(BaseTest):
@@ -600,7 +609,7 @@ class TestReapplyNode(BaseTest):
                 error=self.node_info.error).save(self.session)
 
     def call(self):
-        process._reapply(self.node_info)
+        process._reapply(self.node_info, introspection_data=self.data)
         # make sure node_info lock is released after a call
         self.node_info.release_lock.assert_called_once_with(self.node_info)
 
@@ -616,20 +625,14 @@ class TestReapplyNode(BaseTest):
 
     @prepare_mocks
     def test_ok(self, finished_mock, swift_mock, apply_mock, post_hook_mock):
-        swift_name = 'inspector_data-%s' % self.uuid
-        swift_mock.get_object.return_value = json.dumps(self.data)
-
         self.call()
 
         self.commit_fixture.mock.assert_called_once_with(self.node_info)
 
         post_hook_mock.assert_called_once_with(mock.ANY, self.node_info)
-        swift_mock.create_object.assert_called_once_with(swift_name,
-                                                         mock.ANY)
-        swifted_data = json.loads(swift_mock.create_object.call_args[0][1])
 
         self.node_info.invalidate_cache.assert_called_once_with()
-        apply_mock.assert_called_once_with(self.node_info, swifted_data)
+        apply_mock.assert_called_once_with(self.node_info, self.data)
 
         # assert no power operations were performed
         self.assertFalse(self.cli.node.set_power_state.called)
@@ -637,40 +640,23 @@ class TestReapplyNode(BaseTest):
             self.node_info, istate.Events.finish)
 
         # asserting validate_interfaces was called
-        self.assertEqual(self.pxe_interfaces, swifted_data['interfaces'])
-        self.assertEqual([self.pxe_mac], swifted_data['macs'])
+        self.assertEqual(self.pxe_interfaces, self.data['interfaces'])
+        self.assertEqual([self.pxe_mac], self.data['macs'])
 
         # assert ports were created with whatever there was left
         # behind validate_interfaces
         self.cli.port.create.assert_called_once_with(
             node_uuid=self.uuid,
-            address=swifted_data['macs'][0],
+            address=self.data['macs'][0],
             extra={},
             pxe_enabled=True
         )
 
-    @prepare_mocks
-    def test_get_incomming_data_exception(self, finished_mock, swift_mock,
-                                          apply_mock, post_hook_mock):
-        exc = Exception('Oops')
-        expected_error = ('Unexpected exception Exception while fetching '
-                          'unprocessed introspection data from Swift: Oops')
-        swift_mock.get_object.side_effect = exc
-        self.call()
-
-        self.commit_fixture.mock.assert_called_once_with(self.node_info)
-        self.assertFalse(swift_mock.create_object.called)
-        self.assertFalse(apply_mock.called)
-        self.assertFalse(post_hook_mock.called)
-        finished_mock.assert_called_once_with(
-            self.node_info, istate.Events.error, error=expected_error)
-
     @prepare_mocks
     def test_prehook_failure(self, finished_mock, swift_mock, apply_mock,
                              post_hook_mock):
         CONF.set_override('processing_hooks', 'example',
                           'processing')
-        plugins_base._HOOKS_MGR = None
 
         exc = Exception('Failed.')
         swift_mock.get_object.return_value = json.dumps(self.data)
@@ -692,17 +678,3 @@ class TestReapplyNode(BaseTest):
         self.assertFalse(swift_mock.create_object.called)
         self.assertFalse(apply_mock.called)
         self.assertFalse(post_hook_mock.called)
-
-    @prepare_mocks
-    def test_generic_exception_creating_ports(self, finished_mock, swift_mock,
-                                              apply_mock, post_hook_mock):
-        swift_mock.get_object.return_value = json.dumps(self.data)
-        exc = Exception('Oops')
-        self.cli.port.create.side_effect = exc
-        self.call()
-
-        finished_mock.assert_called_once_with(
-            self.node_info, istate.Events.error, error=str(exc))
-        self.assertFalse(swift_mock.create_object.called)
-        self.assertFalse(apply_mock.called)
-        self.assertFalse(post_hook_mock.called)
